import  Toast from "../toast/index.js"
import { isObeject, isFunction } from "@/utils/types.js"
import Validator from "@/utils/validate.js"
/**
 * @desc 请求 所有的状态码都在这里处理
 * @param { Object } options 参考uni.request参数说明
 * @return {
	status: true / false
	msg
	data
 }
 */
function validateOps(options = {}) {
	const validator = new Validator()
	validator.add(Object.keys(options), [{strategy: 'minLength:1', errorMsg: '参数格式不正确'}])
	validator.add(options.url, [{strategy: 'notEmpty', errorMsg: '请求地址不能为空'}, {strategy: 'isHttp', errorMsg: '请求地址不正确'}])
	return validator.validate()
}

export function request (options) {
    const valite_err = validateOps(options)
	if(valite_err) return Toast({title: valite_err})
	options.method = options.method ? options.method.toUpperCase() : "GET" // 默认get
    return new Promise((resolve, reject) => {
        uni.request({
            ...options,
            header: {},
            success: res => {
                const statusCode = res.statusCode
                // console.log(statusCode)
                switch(statusCode) {
                    case 200:
                        options.successCb && typeof options.successCb === 'function' && options.successCb()
                        // 处理某些接口返回String类型的数据
                        const data = (res.data && typeof res.data === 'string') ? JSON.parse(res.data) : res.data
                        const status = data.stat
                        resolve({
                            status: true,
                            data: data.data,
                            msg: data.msg
                        })
                        break; // 这里的break可以不写，return直接退出函数了
                    case 404:
                        return Toast({title: '请求路径不存在'})                        
                        break;
                    case 500:
                        return Toast({title: '服务器错误'})
                        break;
                    default:
                        Toast({title: '网络请求失败'})
                        return
                        break;    
                }
            },
            fail: err => {
                if(err.errMsg.includes('request:fail ')) {
                    Toast({title: err.errMsg})
                }
                options.errCb && typeof options.errCb === 'function' && options.errCb();
                reject({
                    status: false,
                    code: 400,
                    data: err
                })
            },
            complete: () => {
                options.completeCb && typeof options.completeCb === 'function' && options.completeCb();
            }
        })
    })
}
